/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.VirtualFileSystem.Options;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.SOSOptionBoolean;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
 * \class 		SOSConnection2OptionsAlternate - Options for a connection to an uri (server, site, e.g.)
 *
 * \brief 
 * An Options as a container for the Options super class. 
 * The Option class will hold all the things, which would be otherwise overwritten at a re-creation
 * of the super-class.
 *
 *

 *
 * see \see j:\e\java\development\com.sos.scheduler\src\sos\scheduler\jobdoc\SOSConnection2.xml for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by JobDocu2OptionsClass.xslt from http://www.sos-berlin.com at 20100917112404 
 * \endverbatim
 */
@JSOptionClass(name = "SOSConnection2OptionsAlternate", description = "Options for a connection to an uri (server, site, e.g.)")
@I18NResourceBundle(baseName = "SOSVirtualFileSystem", defaultLocale = "en")
public class SOSConnection2OptionsAlternate extends SOSConnection2OptionsSuperClass {
	/**
	 * 
	 */
	private static final long	serialVersionUID		= 5924032437179660014L;
	@SuppressWarnings("unused")//$NON-NLS-1$
	private final String		conClassName			= "SOSConnection2OptionsAlternate";						//$NON-NLS-1$
	@SuppressWarnings("unused")
	private static Logger		logger					= Logger.getLogger(SOSConnection2OptionsAlternate.class);
	private String				strAlternativePrefix	= "";
	public SOSOptionBoolean		ProtocolCommandListener	= new SOSOptionBoolean(this, conClassName + ".Protocol_Command_Listener", // HashMap-Key
																"use (Apache-) ProtocolCommandListener to log all ftp-commands", // Titel
																"false", // InitValue
																"false", // DefaultValue
																false // isMandatory
														);
	
	/**
	 * \option Post_Command
	 * \type SOSOptionString
	 * \brief Post_Command - FTP-Command to be executed after transfer
	 *
	 * \details
	 * FTP-Command to be executed after transfer for each file.
	 *
	 * \mandatory: false
	 *
	 * \created 12.10.2011 12:28:42 by KB
	 */
	@JSOptionDefinition(name = "Post_Command", description = "FTP-Command to be executed after transfer", key = "Post_Command", type = "SOSOptionString", mandatory = false)
	public SOSOptionString		Post_Command					= new SOSOptionString( // ...
																		this, // ....
																		conClassName + ".Post_Command", // ...
																		"FTP-Command to be executed after transfer", // ...
																		"", // ...
																		"", // ...
																		false);

	public String getPost_Command() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getPost_Command";

		return Post_Command.Value();
	} // public String getPost_Command

	public SOSConnection2OptionsAlternate setPost_Command(final String pstrValue) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setPost_Command";
		Post_Command.Value(pstrValue);
		return this;
	} // public SOSFTPOptions setPost_Command

	/**
	 * \option Pre_Command
	 * \type SOSOptionString  
	 * \brief Pre_Command - A FTP-Command to be execute before transfer of each file is starting
	 *
	 * \details
	 * 
	 *
	 * \mandatory: false
	 *
	 * \created 12.10.2011 12:25:27 by KB
	 */
	@JSOptionDefinition(name = "Pre_Command", description = "FTP-Command to be execute before transfer", key = "Pre_Command", type = "SOSOptionString  ", mandatory = false)
	public SOSOptionString	Pre_Command	= new SOSOptionString( // ...
												this, // ....
												conClassName + ".Pre_Command", // ...
												"", // ...
												"", // ...
												"", // ...
												false);

	public String getPre_Command() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getPre_Command";

		return Pre_Command.Value();
	} // public String getPre_Command

	public SOSConnection2OptionsAlternate setPre_Command(final String pstrValue) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setPre_Command";
		Pre_Command.Value(pstrValue);
		return this;
	} // public SOSFTPOptions setPre_Command

	
	/**
	 * \option FtpS_protocol
	 * \type SOSOptionString
	 * \brief FtpS_protocol - Type of FTPS-Protocol (SSL / TLS)
	 *
	 * \details
	 * Type of FTPS-Protocol, e.g. SSL, TLS
	 *
	 * \mandatory: true
	 *
	 * \created 21.04.2011 18:15:36 by KB
	 */
	@JSOptionDefinition(name = "FtpS_protocol", description = "Type of FTPS-Protocol, e.g. SSL, TLS", key = "FtpS_protocol", type = "SOSOptionString", mandatory = true)
	public SOSOptionString		FtpS_protocol			= new SOSOptionString( // ...
																this, // ....
																conClassName + ".FtpS_protocol", // ...
																"Type of FTPS-Protocol, e.g. SSL, TLS", // ...
																"SSL", // ...
																"SSL", // ...
																true);

	public String getFtpS_protocol() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getFtpS_protocol";
		return FtpS_protocol.Value();
	} // public String getFtpS_protocol

	public SOSConnection2OptionsAlternate setFtpS_protocol(final String pstrValue) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setFtpS_protocol";
		FtpS_protocol.Value(pstrValue);
		return this;
	} // public SOSConnection2OptionsAlternate setFtpS_protocol
	private SOSConnection2OptionsSuperClass	objAlternativeOptions	= new SOSConnection2OptionsSuperClass();

	/**
	* constructors
	*/
	public SOSConnection2OptionsAlternate() {
	} // public SOSConnection2OptionsAlternate

	public SOSConnection2OptionsAlternate(String pstrPrefix) {
		strAlternativePrefix = pstrPrefix;
	} // public SOSConnection2OptionsAlternate

	public SOSConnection2OptionsAlternate(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public SOSConnection2OptionsAlternate

	public SOSConnection2OptionsAlternate(HashMap<String, String> pobjJSSettings) throws Exception {
		super(pobjJSSettings);
		objAlternativeOptions.setAllOptions(pobjJSSettings, strAlternativePrefix + "alternative_");
		this.addProcessedOptions(objAlternativeOptions.getProcessedOptions());
	} // public SOSConnection2OptionsAlternate (HashMap JSSettings)

	public SOSConnection2OptionsAlternate(HashMap<String, String> pobjJSSettings, String pstrPrefix) throws Exception {
		super(pobjJSSettings);
		strAlternativePrefix = pstrPrefix;
		setAllOptions(pobjJSSettings, strAlternativePrefix);
		objAlternativeOptions.setAllOptions(pobjJSSettings, "alternative_" + strAlternativePrefix);
		this.addProcessedOptions(objAlternativeOptions.getProcessedOptions());
	} // public SOSConnection2OptionsAlternate (HashMap JSSettings)

	/**
	 * \brief CheckMandatory - prft alle Muss-Optionen auf Werte
	 *
	 * \details
	 * @throws Exception
	 *
	 * @throws Exception
	 * - wird ausgelst, wenn eine mandatory-Option keinen Wert hat
	 */
	@Override
	// SOSConnection2OptionsAlternateSuperClass
	public void CheckMandatory() {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 * \brief AlternativeOptions
	 *
	 * \details
	 * getter 
	 *
	 * @return the objAlternativeOptions
	 */
	public SOSConnection2OptionsSuperClass Alternatives() {
		return objAlternativeOptions;
	}

	/**
	 * \brief AlternativeOptions - 
	 *
	 * \details
	 * setter 
	 *
	 * @param objAlternativeOptions the value for objAlternativeOptions to set
	 */
	public void AlternativeOptions(SOSConnection2OptionsSuperClass pobjAlternativeOptions) {
		this.objAlternativeOptions = pobjAlternativeOptions;
	}
}
