/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/


package sos.scheduler.InstallationService;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.JSHelper.Listener.JSListenerClass;
import com.sos.JSHelper.Logging.Log4JHelper;

/**
 * \class 		JSBatchInstallerOptionsJUnitTest - Unattended Batch Installation on remote servers
 *
 * \brief 
 *
 *

 *
 * see \see C:\Users\KB\Downloads\Preislisten\JSBatchInstaller.xml for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by C:\Users\KB\eclipse\sos.scheduler.xsl\JSJobDoc2JSJUnitOptionSuperClass.xsl from http://www.sos-berlin.com at 20110322142443 
 * \endverbatim
 *
 * \section TestData Eine Hilfe zum Erzeugen einer HashMap mit Testdaten
 *
 * Die folgenden Methode kann verwendet werden, um fr einen Test eine HashMap
 * mit sinnvollen Werten fr die einzelnen Optionen zu erzeugen.
 *
 * \verbatim
 private HashMap <String, String> SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM) {
	pobjHM.put ("		JSBatchInstallerOptionsJUnitTest.auth_file", "test");  // This parameter specifies the path and name of a user's pr
		return pobjHM;
  }  //  private void SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM)
 * \endverbatim
 */
public class JSBatchInstallerOptionsJUnitTest extends  JSToolBox {
	private final String					conClassName						= "JSBatchInstallerOptionsJUnitTest"; //$NON-NLS-1$
		@SuppressWarnings("unused") //$NON-NLS-1$
	private static Logger		logger			= Logger.getLogger(JSBatchInstallerOptionsJUnitTest.class);
	@SuppressWarnings("unused")
	private static Log4JHelper	objLogger		= null;
	private JSBatchInstaller objE = null;

	protected JSBatchInstallerOptions	objOptions			= null;

	public JSBatchInstallerOptionsJUnitTest() {
		//
	}

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		objLogger = new Log4JHelper("./log4j.properties"); //$NON-NLS-1$
		objE = new JSBatchInstaller();
		objE.registerMessageListener(this);
		objOptions = objE.Options();
		objOptions.registerMessageListener(this);
		
		JSListenerClass.bolLogDebugInformation = true;
		JSListenerClass.intMaxDebugLevel = 9;
	}

	@After
	public void tearDown() throws Exception {
	}


		

/**
 * \brief testconfiguration_path : Path to the folder with the generated installation files.
 * 
 * \details
 * Path to the folder with the generated installation files.
 *
 */
    @Test
    public void testlocal_dir() {  // SOSOptionFolderName
    	objOptions.local_dir.Value("++----++");
    	assertEquals ("Path to the folder with the generated installation files.", objOptions.local_dir.Value(),"++----++");
    	
    }

                

/**
 * \brief testexecute_all_entries : False: Only installations are executed which have no entry "LastRun"
 * 
 * \details
 * False: Only installations are executed which have no entry "LastRun" True: All installations are executed (matching a given filter).
 *
 */
    @Test
    public void testexecute_all_entries() {  // SOSOptionBoolean
    	 objOptions.update.Value("true");
    	 assertTrue ("False: Ignore value of LastRun", objOptions.update.value());
    	 objOptions.update.Value("false");
    	 assertFalse ("False: Ignore value of LastRun", objOptions.update.value());
    	
    }

                

/**
 * \brief testfilter_install_host : Only installations are executed which belongs to this host.
 * 
 * \details
 * Only installations are executed which belongs to this host.
 *
 */
    @Test
    public void testfilter_install_host() {  // SOSOptionHostName
    	objOptions.filter_install_host.Value("++----++");
    	assertEquals ("Only installations are executed which belongs to this host.", objOptions.filter_install_host.Value(),"++----++");
    	
    }

                

/**
 * \brief testfilter_install_port : Only installations are executed which belongs to this port.
 * 
 * \details
 * Only installations are executed which belongs to this port.
 *
 */
    @Test
    public void testfilter_install_port() {  // SOSOptionPortNumber
    	objOptions.filter_install_port.Value("++0++");
    	assertEquals ("Only installations are executed which belongs to this port.", objOptions.filter_install_port.Value(),"++0++");
    	
    }

                

/**
 * \brief testinstallation_definition_file : XML file with installation elements. One element per installation.
 * 
 * \details
 * XML file with installation elements. One element per installation.
 *
 */
    @Test
    public void testinstallation_definition_file() {  // SOSOptionInFileName
    	objOptions.installation_definition_file.Value("++----++");
    	assertEquals ("XML file with installation elements. One element per installation.", objOptions.installation_definition_file.Value(),"++----++");
    	
    }

                

/**
 * \brief testinstallation_job_chain : Job chain with the steps for transfer the installation files and perfo
 * 
 * \details
 * Job chain with the steps for transfer the installation files and perform the setup. The job chain must contain the nodes: -TransferInstallationSetup (FTP) Transfering setup -TransferInstallationFile (FTP) Transfering installation file -PerformInstall (SSH) Executing setup
 *
 */
    @Test
    public void testinstallation_job_chain() {  // JSOptionJobChainName
    	objOptions.installation_job_chain.Value("++automatic_installation++");
    	assertEquals ("Job chain with the steps for transfer the installation files and perfo", objOptions.installation_job_chain.Value(),"++automatic_installation++");
    	
    }

                
        
} // public class JSBatchInstallerOptionsJUnitTest
